<?php
/**
 * @copyright Copyright (C) 2024, Salvatore Noschese
 *
 * Plugin Name:       MustUse Manager (Lite)
 * Plugin URI:        https://salvatorenoschese.it/mustuse-manager-wordpress-plugin/
 * Description:       Download, Activate, Deactivate or Delete mu-plugins from the WordPress backend.
 * Version:           1.0
 * Author:            Salvatore Noschese
 * Author URI:        https://salvatorenoschese.it/
 * Text Domain:       mustuse-manager
 * Domain Path:       /languages
 * License:           GPL-2.0-or-later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 * Requires at least: 6.0
 * Requires PHP:      8.0
 */

/* Prevent direct access */
defined('ABSPATH') || exit;

/**
 * Class properties for MustUse Manager.
 * notify output is escaped in display_admin_notices()
 */
class MustUSE_Manager
{
	/** @var string The directory path for MU-Plugins */
	private $mu_plugins_dir;

	/** @var array Data about the current plugin */
	private $plugin_data;

	/**
	 * Constructor: Initializes the MustUse Manager plugin.
	 *
	 * This constructor sets up essential plugin parameters, registers actions and filters,
	 * and loads necessary resources to ensure the plugin functions correctly.
	 */
	public function __construct()
	{
		// Ensure the get_plugin_data function is available (it's part of the WordPress admin API)
		if (!function_exists('get_plugin_data')) {
			require_once(ABSPATH . 'wp-admin/includes/plugin.php');
		}

		// Initialize WP_Filesystem if it hasn't been initialized yet
		global $wp_filesystem;
		if (empty($wp_filesystem)) {
			require_once(ABSPATH . '/wp-admin/includes/file.php');
			WP_Filesystem();
		}

		// Assign the global $wp_filesystem
		$this->wpfilesys = $wp_filesystem;

		// Set the path and url to the must-use directory
		$this->mu_plugins_dir = WPMU_PLUGIN_DIR;
		$this->mu_download_dir = WPMU_PLUGIN_DIR . '/_download/';
		$this->mu_download_url = WPMU_PLUGIN_URL . '/_download/';

		// Retrieve this plugin's metadata
		$this->plugin_data = get_plugin_data(__FILE__);

		// Set the MustUse Manager URL as a class variable
		$this->mustuse_manager_url = self_admin_url('plugins.php?page=mustuse-manager');

		// Handle plugin actions in 'admin_init' to prevent 'headers already sent' warnings
		add_action('admin_init', function () {
			$this->handle_actions();      // activating, deactivating, download, deleting
			$this->handle_bulk_actions(); // bulk actions
		});

		// Add custom button to manage MU-plugins in the WordPress admin menu
		add_action('admin_menu', [$this, 'add_manage_button']);
		add_action('admin_enqueue_scripts', function () { // And hide it via CSS + Redirect */plugins.php?plugin_status=mustuse to */plugins.php?page=mustuse-manager
			wp_enqueue_style('mustuse-manager-hidemenu', plugins_url('assets/mustuse-manager-hide-menu.css', __FILE__), array(), $this->plugin_data['Version']);
			wp_enqueue_script('mustuse-manager-redirect', plugins_url('assets/mustuse-manager-redirect.js', __FILE__), array('jquery'), $this->plugin_data['Version'], true);
		});

		// Modifies the URL for the Must-Use tab in the plugins page.
		add_filter('views_plugins', function ($views) {
			// If the 'mustuse' view exists, modify only the URL
			if (isset($views['mustuse'])) {
				$views['mustuse'] = preg_replace('/href=(["\']).*?\\1/', 'href="' . esc_url($this->mustuse_manager_url) . '"', $views['mustuse']);
			} else { // if no muplugin, set to 0 and use plural for translation
				$views['mustuse'] = '<a href="' . esc_url($this->mustuse_manager_url) . '">' . esc_html__('Must-Use', 'mustuse-manager') . ' <span class="count">(0)</span></a>';
			}
			return $views;
		});

		// Add custom actions/links (e.g., donate link) to the plugin row in the plugins.php page
		add_filter('plugin_action_links_' . plugin_basename(__FILE__), [$this, 'add_plugin_page_settings_link']);
		add_filter('plugin_row_meta', [$this, 'plugin_row_meta'], 10, 2);

		// Customize the footer text on the MustUse Manager page
		add_filter('admin_footer_text', [$this, 'custom_footer_text']);

		// Load the text domain for internationalization, enabling translations
		load_plugin_textdomain('mustuse-manager', false, dirname(plugin_basename(__FILE__)) . '/languages');
	}

	/**
	 * Adds the "MustUse Manager" button to the Plugins menu in the WordPress admin.
	 *
	 * This function:
	 * - Adds a submenu page under the "Plugins" section in the WordPress dashboard.
	 * - Even though the button will be hidden via CSS, adding it ensures that the "Plugins" menu remains active when
	 *   the "MustUse Manager" page is accessed.
	 * - Note: Access to the "MustUse Manager" page is typically done via the must-use plugins URL, which is already present in WordPress.
	 * - Enqueues specific styles and scripts needed only on the "MustUse Manager" page.
	 */
	public function add_manage_button()
	{
		// Adds a new submenu under "Plugins" in the admin menu
		// The page is linked to the 'mustuse-manager' slug and is accessible to users with 'manage_options' capability
		$hook = add_submenu_page(
			'plugins.php',             // The parent page in the menu (the default "Plugins" page)
			'MustUse Manager',         // The page title displayed in the browser tab
			'MustUse Manager',         // The title shown in the menu (hidden via css)
			'manage_options',          // The required capability to access this page
			'mustuse-manager',         // The unique slug for this menu page
			[$this, 'management_page'] // Callback function to render the content of the page
		);

		// Enqueues styles and scripts specifically for the "MustUse Manager" page when it is loaded
		// The button is hidden via CSS, but this ensures the "Plugins" menu appears as active
		add_action("admin_print_styles-{$hook}", function () {
			wp_enqueue_style('mustuse-manager-styles', plugins_url('assets/mustuse-manager-styles.css', __FILE__), array(), $this->plugin_data['Version']);
			wp_enqueue_script('mustuse-manager-script', plugins_url('assets/mustuse-manager-script.js', __FILE__), array('jquery'), $this->plugin_data['Version'], true);
			wp_localize_script('mustuse-manager-script', 'muPluginL10n', array(
				'deleteConfirmation' => esc_js(__('Are you sure you want to delete this Mu-Plugin?', 'mustuse-manager')),
				/* translators: %s: number of selected mu-plugins */
				'bulkDeleteMultipleConfirm' => esc_js(__('Are you sure you want to delete the selected Mu-Plugins (%s)?', 'mustuse-manager')),
				/* translators: %s: number of selected mu-plugins */
				'bulkDeleteAllConfirm' => esc_js(__('Are you sure you want to delete all selected Mu-Plugins (%s)?', 'mustuse-manager'))
			));
		});
	}

	/**
	 * Redirects to the MustUse Manager page.
	 * And exit to prevent further code execution.
	 */
	private function redirect_to_mustuse_manager()
	{
		// Perform a safe redirect
		wp_safe_redirect(esc_url($this->mustuse_manager_url));
		exit;
	}

	/**
	 * Displays the main management page for MU-Plugins.
	 * This function handles the overall layout and logic for the MU-Plugins management interface.
	 */
	public function management_page()
	{
		// Verify that the mu-plugins directory exists, if not, attempt to create it
		$page_title = '<h1>MustUse Manager</h1>';
		$support_link = '<p>' . sprintf(
			/* translators: %s: = support url */
			esc_html__('For support, please visit: %s', 'mustuse-manager'),
			'<a href="' . esc_url($this->plugin_data['PluginURI']) . '" target="_blank">' . esc_html__('Support', 'mustuse-manager') . '</a>'
		) . '</p>';
		if (!$this->wpfilesys->exists($this->mu_plugins_dir)) {
			// Attempt to create the directory and handle failure
			if (!$this->wpfilesys->mkdir($this->mu_plugins_dir)) {
				wp_die(
					wp_kses_post($page_title) .
					'<p>' . esc_html__('Could not create mu-plugins directory. Please check your permissions.', 'mustuse-manager') . '</p>' .
					wp_kses_post($support_link)
				);
			}
			// Notify admin if the directory was successfully created
			$this->add_admin_notice(esc_html__('The mu-plugins directory was successfully created.', 'mustuse-manager'), 'info');
		} elseif (!$this->wpfilesys->is_writable($this->mu_plugins_dir)) {
			// If the directory exists but is not writable, terminate and notify the user
			wp_die(
				wp_kses_post($page_title) .
				'<p>' . esc_html__('The mu-plugins directory is not writable. Please check your server configuration.', 'mustuse-manager') . '</p>' .
				wp_kses_post($support_link)
			);
		}

		// Check if the download directory exists
		if ($this->wpfilesys->exists($this->mu_download_dir)) {
			// If it exists, delete all files in the directory
			$files = glob($this->mu_download_dir . '*'); // Get all files in the directory
			foreach ($files as $file) {
				if (is_file($file)) { 
					$this->wpfilesys->delete($file); // Delete each file
				}
			}
		} else {
			// If the directory does not exist, create it
			$this->wpfilesys->mkdir($this->mu_download_dir);
		}

		// Check for an existing index.php file in the mu-plugins directory
		if ($this->wpfilesys->exists($this->mu_plugins_dir . '/index.php')) {
			// Rename the existing index.php file to avoid conflicts
			$new_name = 'index.php-' . time();
			if ($this->wpfilesys->move($this->mu_plugins_dir . '/index.php', $this->mu_plugins_dir . '/' . $new_name, true)) {
				/* translators: %s: new filename for the renamed index.php */
				$this->add_admin_notice(sprintf(__('Existing index.php found and renamed to %s.', 'mustuse-manager'), $new_name), 'warning');
			} else {
				// Notify admin if the renaming process fails
				$this->add_admin_notice(__('Failed to rename existing index.php.', 'mustuse-manager'), 'error');
			}
		}

		// Create an empty index.html file if it doesn't exist to prevent directory listing
		if (!$this->wpfilesys->exists($this->mu_plugins_dir . '/index.html')) {
			// Attempt to create the index.html file and handle success or failure
			if ($this->wpfilesys->put_contents($this->mu_plugins_dir . '/index.html', "", FS_CHMOD_FILE)) {
				$this->add_admin_notice(__('Created an empty index.html in the mu-plugins directory.', 'mustuse-manager'), 'info');
			} else {
				$this->add_admin_notice(__('Failed to create index.html in the mu-plugins directory.', 'mustuse-manager'), 'error');
			}
		}

		// Scan the mu-plugins directory and filter only files with a 'php' or 'inactive' extension, excluding 'index.php'.
		$files = array_filter(scandir($this->mu_plugins_dir), function ($file) {
			// Get the file extension to filter for 'php' or 'inactive' files.
			$extension = pathinfo($file, PATHINFO_EXTENSION);
			return in_array($extension, ['php', 'inactive']) && $file !== 'index.php';
		});

		// Map the filtered files to return an array with the file name, plugin name, and status (active or inactive).
		$plugins = array_map(function ($file) {
			// Remove the extension (either '.php' or '.inactive') from the file name to get the plugin name.
			$plugin_name = str_replace(['.php', '.inactive'], '', $file);
			// Determine the plugin status based on the file extension: 'inactive' if the extension is '.inactive', otherwise 'active'.
			$status = (pathinfo($file, PATHINFO_EXTENSION) === 'inactive') ? 'inactive' : 'active';
			return ['file' => $file, 'name' => $plugin_name, 'status' => $status];
		}, $files);

		// Pre-process plugin data
		foreach ($plugins as &$plugin) {
			$plugin_data = get_plugin_data($this->mu_plugins_dir . '/' . $plugin['file']);
			$plugin['display_name'] = !empty($plugin_data['Name']) ? $plugin_data['Name'] : $plugin['name'];
		}
		unset($plugin); // Unset reference to last element

		// Sort plugins
		usort($plugins, function ($a, $b) {
			return strcasecmp($a['display_name'], $b['display_name']);
		});

		// Count active and inactive plugins
		$count_active = $count_inactive = 0;
		foreach ($plugins as $plugin) {
			$plugin['status'] === 'active' ? $count_active++ : $count_inactive++;
		}

		// Start HTML output
		echo '<div class="wrap">';
		echo '<div class="header-container">';
		echo '<h1 class="wp-heading-inline">MustUse Manager</h1>'; // no need to translate
		echo '<a class="page-title-action" href="' . esc_url(self_admin_url('plugins.php')) . '">' . esc_html__('Installed Plugins', 'mustuse-manager') . '<span class="dashicons dashicons-admin-plugins"></span></a>';
		echo '</div>';
		echo '<hr class="wp-header-end" />';

		// Display admin notices
		$this->display_admin_notices();

		// Fallback message for disabled JavaScript
		echo '<div class="hide-if-js">';
		echo '<h3>' . esc_html__('Sorry, cannot work without JavaScript', 'mustuse-manager') . '</h3>';
		echo '</div>';

		// Display a message if no MU-Plugins are found
		if (empty($plugins)) {
			echo '<h3>' . esc_html__('No MU-Plugin found. Add your first one.', 'mustuse-manager') . '</h3>';
		} else {
			// Display MU-Plugins statistics
			echo '<p class="mu-plugin-stats">';
			$total_plugins = count($plugins);
			if ($total_plugins === 1) {
				// Add message for active plugins
				echo '<span class="all">' . esc_html__('Found 1 MU-Plugin', 'mustuse-manager') . '</span>: ';
				// Handle the case when there's only one MU-Plugin
				if ($count_active === 1) {
					echo '<span class="active">' . esc_html__('It is active.', 'mustuse-manager') . '</span>';
				} else {
					echo '<span class="inactive">' . esc_html__('It is inactive.', 'mustuse-manager') . '</span>';
				}
			} else {
				// translators: %d is the number of MU-Plugins
				echo '<span class="all">' . esc_html(sprintf(__('Found %d MU-Plugins', 'mustuse-manager'), $total_plugins)) . '</span>: ';
				// Handle the case when there are multiple MU-Plugins
				if ($count_active === $total_plugins) {
					// All MU-Plugins are active
					echo '<span class="active">' . esc_html__('All are active.', 'mustuse-manager') . '</span>';
				} elseif ($count_inactive === $total_plugins) {
					// All MU-Plugins are inactive
					echo '<span class="inactive">' . esc_html__('All are inactive.', 'mustuse-manager') . '</span>';
				} else {
					// Some MU-Plugins are active, some are inactive
					$messages = array();
					if ($count_active > 0) {
						// Add message for active plugins
						if ($count_active === 1) {
							$messages[] = '<span class="active">' . esc_html__('1 is active', 'mustuse-manager') . '</span>';
						} else {
							/* translators: %d is the number of active MU-Plugins */
							$messages[] = '<span class="active">' . esc_html(sprintf(__('%d are active', 'mustuse-manager'), $count_active)) . '</span>';
						}
					}
					if ($count_inactive > 0) {
						// Add message for inactive plugins
						if ($count_inactive === 1) {
							$messages[] = '<span class="inactive">' . esc_html__('1 is inactive', 'mustuse-manager') . '</span>';
						} else {
							/* translators: %d is the number of inactive MU-Plugins */
							$messages[] = '<span class="inactive">' . esc_html(sprintf(__('%d are inactive', 'mustuse-manager'), $count_inactive)) . '</span>';
						}
					}
					// Join messages with a comma
					echo wp_kses_post(implode(', ', $messages) . '.');
				}
			}
			echo '</p>'; // end mu-plugins stats

			// Start of plugins table
			echo '<form method="post" id="bulk-action-form">';
			wp_nonce_field('mustuse_manager_bulk_action', '_wpnonce_bulk');

			// Bulk actions
			$this->display_bulk_actions();

			// Plugins table
			echo '<table class="wp-list-table widefat plugins">';
			$this->display_table_section();
			echo '<tbody id="the-list">';
			foreach ($plugins as $plugin_name => $plugin_info) {
				$this->display_plugin_row($plugin_name, $plugin_info);
			}
			echo '</tbody>';
			$this->display_table_section('footer');
			echo '</table>';

			// Bottom bulk actions
			$this->display_bulk_actions('bottom');

			echo '</form>';
		}

		// End HTML output
		echo '</div>';
	}

	/**
	 * Displays the bulk actions dropdown menu.
	 * This function creates a dropdown menu for bulk actions like activate, deactivate, and delete.
	 *
	 * @param string $position The position of the dropdown (top or bottom).
	 */
	private function display_bulk_actions($position = 'top')
	{
		$id_suffix = ($position === 'bottom') ? '2' : '';
		echo '<div class="tablenav ' . esc_attr($position) . '">';
		echo '<div class="alignleft actions bulkactions">';
		echo '<label for="bulk-action-selector-' . esc_attr($position) . '" class="screen-reader-text">' . esc_html__('Select bulk action', 'mustuse-manager') . '</label>';
		echo '<select name="action' . esc_attr($id_suffix) . '" id="bulk-action-selector-' . esc_attr($position) . '" disabled>';
		echo '<option value="-1">' . esc_html__('Bulk actions', 'mustuse-manager') . '</option>';
		// check activate/deactivate permission
		if (current_user_can('activate_plugins')) {
			echo '<option value="activate-selected">' . esc_html__('Activate', 'mustuse-manager') . '</option>';
			echo '<option value="deactivate-selected">' . esc_html__('Deactivate', 'mustuse-manager') . '</option>';
		} else {
			echo '<option disabled>' . esc_html__('Activate', 'mustuse-manager') . '</option>';
			echo '<option disabled>' . esc_html__('Deactivate', 'mustuse-manager') . '</option>';
		}
		// check delete permission
		if (current_user_can('delete_plugins')) {
			echo '<option value="delete-selected">' . esc_html__('Delete', 'mustuse-manager') . '</option>';
		} else {
			echo '<option disabled>' . esc_html__('Delete', 'mustuse-manager') . '</option>';
		}
		echo '</select>';
		echo '<input type="submit" id="doaction' . esc_attr($id_suffix) . '" class="button action" value="' . esc_html__('Apply', 'mustuse-manager') . '" disabled>';
		echo '</div>';
		echo '<br class="clear" />';
		echo '</div>';
	}

	/**
	 * Displays the table header or footer.
	 * This function generates the HTML for the table header or footer, including column titles.
	 *
	 * @param string $position Whether to display the header or footer.
	 */
	private function display_table_section($position = 'header')
	{
		$tag = ($position === 'header') ? 'thead' : 'tfoot';
		$idcb = ($position === 'header') ? ' id="cb"' : '';
		$checkbox_id = ($position === 'header') ? 'cb-select-all-1' : 'cb-select-all-2';
		echo '<' . esc_attr($tag) . '>';
		echo '<tr>';
		echo '<td' . esc_attr($idcb) . ' class="manage-column column-cb check-column">';
		echo '<input id="' . esc_attr($checkbox_id) . '" type="checkbox" class="hide-if-no-js" />';
		echo '<label for="' . esc_attr($checkbox_id) . '"><span class="screen-reader-text">' . esc_html__('Select All', 'mustuse-manager') . '</span></label>';
		echo '</td>';
		echo '<th scope="col" id="name" class="manage-column column-name column-primary">' . esc_html__('Mu-Plugin', 'mustuse-manager') . '</th>';
		echo '<th scope="col" id="description" class="manage-column column-description">' . esc_html__('Description', 'mustuse-manager') . '</th>';
		echo '<th scope="col" id="auto-updates" class="manage-column column-auto-updates">' . esc_html__('Actions', 'mustuse-manager') . '</th>';
		echo '</tr>';
		echo '</' . esc_attr($tag) . '>';
	}

	/**
	 * Displays a single plugin row in the table.
	 * This function generates the HTML for a single plugin row, including plugin details and action links.
	 *
	 * @param string $plugin_name The name of the plugin.
	 * @param array $plugin_info An array containing plugin information.
	 */
	private function display_plugin_row($plugin_name, $plugin_info)
	{
		// Get plugin file and status
		$plugin_file = $plugin_info['file'];
		$status = $plugin_info['status'];

		// Construct plugin path and get plugin data
		$plugin_path = $this->mu_plugins_dir . '/' . $plugin_file;
		$plugin_data = get_plugin_data($plugin_path, false);

		// Prepare plugin display name and version
		$plugin_filename = str_replace('.inactive', '', $plugin_file);
		$plugin_name_display = !empty($plugin_data['Name']) ? $plugin_data['Name'] : ucwords(str_replace('.php', '', $plugin_filename));

		// Set plugin description
		$plugin_description = $plugin_data['Description'];

		// Generate plugin meta data (Version, Author, Plugin URI)
		$plugin_meta = array();
		if (!empty($plugin_data['Version'])) { /* translators: %s: VERSION NUMBER */
			$plugin_meta[] = sprintf(esc_html__('Version %s', 'mustuse-manager'), $plugin_data['Version']);
		}

		if (!empty($plugin_data['Author'])) {
			$author = esc_html($plugin_data['Author']);
			if (!empty($plugin_data['AuthorURI'])) {
				$author = '<a href="' . esc_url($plugin_data['AuthorURI']) . '" title="' . esc_attr__('Visit author homepage', 'mustuse-manager') . '">' . esc_html($plugin_data['Author']) . '</a>';
			}
			/* translators: %s: author */
			$plugin_meta[] = sprintf(esc_html__('By %s', 'mustuse-manager'), $author);
		}

		if (!empty($plugin_data['PluginURI'])) {
			$plugin_meta[] = '<a href="' . esc_url($plugin_data['PluginURI']) . '" title="' . esc_attr__('Visit plugin site', 'mustuse-manager') . '">' . esc_html__('Visit plugin site', 'mustuse-manager') . '</a>';
		}

		// Apply filters to plugin meta and join with ' | '
		$plugin_meta = apply_filters('plugin_row_meta', $plugin_meta, $plugin_file, $plugin_data, $status);
		$meta_html = !empty($plugin_meta) ? implode(' | ', $plugin_meta) : '';

		// Get additional action links using filters
		$plugin_basename = plugin_basename($plugin_path);
		$plugin_action_links = apply_filters('plugin_action_links_' . $plugin_basename, array());
		$links_html = !empty($plugin_action_links) ? ': ' . implode(' | ', $plugin_action_links) : '';

		// Output plugin row: no translate
		echo '<tr class="' . esc_attr($status) . '">';
		echo '<th scope="row" class="check-column">';
		echo '<label class="label-covers-full-cell" for="' . esc_attr($plugin_file) . '">';
		/* translators: %s: plugin_name_display */
		echo '<span class="screen-reader-text">' . sprintf(esc_html__('Select %s', 'mustuse-manager'), esc_attr($plugin_name_display)) . '</span>';
		echo '</label>';
		echo '<input type="checkbox" name="checked[]" value="' . esc_attr($plugin_file) . '" id="' . esc_attr($plugin_file) . '" class="hide-if-no-js"/>';
		echo '</th>';

		// Plugin name, flag, and filename (with additional links)
		echo '<td class="plugin-title column-primary">';
		echo '<strong>' . esc_html($plugin_name_display) . '</strong>';
		echo '<small>' . esc_html($plugin_filename) . '' . esc_html($links_html) . '</small>';
		echo '</td>';

		// Plugin description, followed by version and author info
		echo '<td class="column-description desc">';
		echo '<div class="plugin-description"><p>' . esc_html($plugin_description) . '</p></div>';
		if (!empty($meta_html)) {
			echo '<div class="plugin-versionauthor"><small>' . wp_kses_post($meta_html) . '</small></div>';
		}
		echo '</td>';

		// Determine the action and its label based on the current plugin status
		$action = ($status === "active") ? 'deactivate' : 'activate';
		$action_text = ($status === "active") ? __('Deactivate', 'mustuse-manager') : __('Activate', 'mustuse-manager');

		// Generate the base URL for all plugin actions (activate, deactivate, delete, download)
		$base_url = add_query_arg([
			'page' => 'mustuse-manager',                                     // The page for managing mu-plugins
			'plugin' => urlencode($plugin_file),                             // The plugin file being acted upon
			'_wpnonce_actions' => wp_create_nonce('mustuse-manager-action'), // Nonce for security
		], self_admin_url('plugins.php'));                                   // Base admin URL

		// Append specific actions to the base URL
		$action_url = add_query_arg('action', $action, $base_url);      // URL for activating or deactivating the plugin
		$delete_url = add_query_arg('action', 'delete', $base_url);     // URL for deleting the plugin
		$download_url = add_query_arg('action', 'download', $base_url); // URL for downloading the plugin

		// Output the action links (activate/deactivate, delete, download) in the table row
		echo '<td class="column-auto-updates">';
		echo '<div class="actions-container hide-if-no-js">';
		if (current_user_can('activate_plugins')) {
			echo '<a class="action-link" href="' . esc_url($action_url) . '">' . esc_html($action_text) . '</a>'; // Activate/Deactivate link
		}
		else {
			echo '<span class="action-link">' . esc_html($action_text) . '</span>'; // Fake Activate/Deactivate button
		}
		echo '<a class="download-link" href="' . esc_url($download_url) . '">' . esc_html__('Download', 'mustuse-manager') . '</a>'; // Download link
		if (current_user_can('delete_plugins')) {
			echo '<a class="delete-link" href="' . esc_url($delete_url) . '">' . esc_html__('Delete', 'mustuse-manager') . '</a>'; // Delete link
		}
		else {
			echo '<span class="delete-link">' . esc_html__('Delete', 'mustuse-manager') . '</span>'; // Fake Delete button
		}
		echo '</div>';
		echo '</td>';
		echo '</tr>';
	}

	/**
	 * Handles single plugin actions.
	 * This function processes actions like activate, deactivate, delete, and download for individual plugins.
	 */
	private function handle_actions()
	{
		// Check if action and plugin are set and verify the nonce
		if (!isset($_GET['page'], $_GET['action'], $_GET['plugin']) 
			|| $_GET['page'] !== 'mustuse-manager'
			|| !check_admin_referer('mustuse-manager-action', '_wpnonce_actions')) {
			return;
		}

		// Sanitize the action and plugin parameters
		$action = sanitize_text_field(wp_unslash($_GET['action']));
		$plugin = urldecode(sanitize_text_field(wp_unslash($_GET['plugin'])));
		$plugin = str_replace('.inactive', '', $plugin);
		$plugin_path = $this->mu_plugins_dir . '/' . $plugin;

		switch ($action) {
			case 'deactivate':
				// Disable the plugin by renaming it
				if (current_user_can('activate_plugins')) {
					$this->wpfilesys->move($plugin_path, $plugin_path . '.inactive');
					/* translators: %s: plugin */
					$this->add_admin_notice(sprintf(__('%s is now inactive', 'mustuse-manager'), $plugin), 'info notice-inactive');
				} else {
					$this->add_admin_notice(sprintf(__('Sorry, you are not allowed to deactivate mu-plugins.', 'mustuse-manager'), $plugin), 'error');
				}
				break;

			case 'activate':
				// Enable the plugin by renaming it back
				if (current_user_can('activate_plugins')) {
					$this->wpfilesys->move($plugin_path . '.inactive', $plugin_path);
					/* translators: %s: plugin */
					$this->add_admin_notice(sprintf(__('%s is now active', 'mustuse-manager'), $plugin), 'success notice-active');
				} else {
					$this->add_admin_notice(sprintf(__('Sorry, you are not allowed to activate mu-plugins.', 'mustuse-manager'), $plugin), 'error');
				}
				break;

			case 'delete':
				// Delete the plugin file
				if (current_user_can('delete_plugins')) {
					$this->wpfilesys->delete($plugin_path);
					$this->wpfilesys->delete($plugin_path . '.inactive');
					/* translators: %s: plugin */
					$this->add_admin_notice(sprintf(__('%s has been deleted', 'mustuse-manager'), $plugin), 'warning notice-delete');
				} else { // just for extra security
					$this->add_admin_notice(sprintf(__('Sorry, you are not allowed to delete mu-plugins.', 'mustuse-manager'), $plugin), 'error');
				}
				break;

			case 'download':
				// Create the zip file in the mu-plugins directory
				$zip_filename = basename($plugin) . '_' . uniqid() . '.zip';
				$zip_file = $this->mu_download_dir . $zip_filename;
				$file_path = $this->wpfilesys->exists($plugin_path) ? $plugin_path : $plugin_path . '.inactive';

				if ($this->wpfilesys->exists($file_path)) {
					$zip = new ZipArchive();
					// Attempt to create the zip file
					if ($zip->open($zip_file, ZipArchive::CREATE) !== true) {
						$this->add_admin_notice(__('Failed to create zip file', 'mustuse-manager'), 'error');
						break;
					}

					// Add the file to the zip archive
					$zip->addFile($file_path, basename($file_path)); // Use basename for the zip
					$zip->close();

					// Redirect to the download URL
					wp_safe_redirect($this->mu_download_url . $zip_filename);
					exit; // Ensure no further code is executed
				} else { // Handle error if the file does not exist
					/* translators: %s: plugin */
					$this->add_admin_notice(sprintf(__('Failed to download Mu-Plugin: %s (file not found)', 'mustuse-manager'), $plugin), 'error');
				}
				break;

			default:
				// Handle unknown actions
				$this->add_admin_notice(__('Unknown action', 'mustuse-manager'), 'error');
				break;
		}

		// Redirect at the end
		$this->redirect_to_mustuse_manager();
	}

	/**
	 * Handles bulk actions for multiple plugins.
	 * This function processes bulk actions like activate, deactivate, and delete for multiple selected plugins.
	 */
	private function handle_bulk_actions()
	{
		// Check if the bulk action form is submitted and valid
		if (!isset($_GET['page']) || $_GET['page'] !== 'mustuse-manager' 
			|| !isset($_POST['action']) || $_POST['action'] == '-1' 
			|| !isset($_POST['checked']) || !check_admin_referer('mustuse_manager_bulk_action', '_wpnonce_bulk')) {
			return;
		}

		// Sanitize and store the selected action and checked plugins
		$action = sanitize_text_field(wp_unslash($_POST['action']));
		$checked_plugins = array_map('sanitize_text_field', wp_unslash($_POST['checked']));
		$success_count = 0; // Track the number of successful actions

		// Define messages for each action (deactivate, activate, delete)
		$action_messages = [
			'deactivate-selected' => [
				'singular' => __('1 Mu-Plugin is now deactivated', 'mustuse-manager'),
				/* translators: %d: number of deactivated plugins */
				'plural'   => __('%d Mu-Plugins are now deactivated', 'mustuse-manager')
			],
			'activate-selected' => [
				'singular' => __('1 Mu-Plugin is now activated', 'mustuse-manager'),
				/* translators: %d: number of activated plugins */
				'plural'   => __('%d Mu-Plugins are now activated', 'mustuse-manager')
			],
			'delete-selected' => [
				'singular' => __('1 Mu-Plugin has been deleted', 'mustuse-manager'),
				/* translators: %d: number of deleted plugins */
				'plural'   => __('%d Mu-Plugins have been deleted', 'mustuse-manager')
			]
		];

		// Loop through each checked plugin and perform the appropriate action
		foreach ($checked_plugins as $plugin) {
			// Remove '.inactive' from the plugin name if it exists
			$plugin = str_replace('.inactive', '', $plugin);
			$plugin_path = $this->mu_plugins_dir . '/' . $plugin;

			switch ($action) {
				case 'deactivate-selected':
					// Deactivate the plugin by renaming it with a '.inactive' suffix
					if (current_user_can('activate_plugins')) {
						if ($this->wpfilesys->exists($plugin_path) && $this->wpfilesys->move($plugin_path, $plugin_path . '.inactive')) {
							$success_count++;
						}
					} else { // just for extra security
						$this->add_admin_notice(sprintf(__('Sorry, you are not allowed to deactivate mu-plugins.', 'mustuse-manager'), $plugin), 'error');
					}
					break;

				case 'activate-selected':
					// Activate the plugin by removing the '.inactive' suffix
					if (current_user_can('activate_plugins')) {
						if ($this->wpfilesys->exists($plugin_path . '.inactive') && $this->wpfilesys->move($plugin_path . '.inactive', $plugin_path)) {
							$success_count++;
						}
					} else { // just for extra security
						$this->add_admin_notice(sprintf(__('Sorry, you are not allowed to activate mu-plugins.', 'mustuse-manager'), $plugin), 'error');
					}
					break;

				case 'delete-selected':
					// Delete the plugin file (whether it's active or inactive)
					if (current_user_can('delete_plugins')) {
						$file_to_delete = $this->wpfilesys->exists($plugin_path . '.inactive') ? $plugin_path . '.inactive' : $plugin_path;
						if ($this->wpfilesys->exists($file_to_delete) && $this->wpfilesys->delete( $file_to_delete ) ) {
							$success_count++;
						}
					} else { // just for extra security
						$this->add_admin_notice(sprintf(__('Sorry, you are not allowed to delete mu-plugins.', 'mustuse-manager'), $plugin), 'error');
					}
					break;
			}
		}

		// If any actions were successful, display the appropriate success message
		if ($success_count > 0) {
			$notice_type = ($action === 'delete-selected') ? 'warning notice-delete' : ($action === 'activate-selected' ? 'success notice-active' : 'info notice-inactive');
			$message_key = $success_count === 1 ? 'singular' : 'plural';
			$message = sprintf($action_messages[$action][$message_key], $success_count);
			$this->add_admin_notice($message, $notice_type);
		} else {
			// If no changes were made, show a neutral notice
			$this->add_admin_notice(__('Nothing changed', 'mustuse-manager'), 'info notice-nochange');
		}

		// Redirect to the MustUSE Manager page after processing the bulk actions
		$this->redirect_to_mustuse_manager();
	}

	/**
	 * Adds settings and support links to the plugin's entry in the plugins list.
	 * This function adds additional links to the plugin's row on the WordPress plugins page.
	 *
	 * @param array $actions The existing array of plugin actions.
	 * @return array The modified array of plugin actions.
	 */
	public function add_plugin_page_settings_link($actions)
	{
		// Create new action links for Support and Settings: no translate
		$mylinks = [
			'<a href="' . esc_url($this->plugin_data['PluginURI']) . '" title="' . esc_attr__('Support', 'mustuse-manager') . '" target="_blank">' . esc_html__('Support', 'mustuse-manager') . '</a>',
			'<a href="' . esc_url($this->mustuse_manager_url) . '" title="' . esc_attr__('Settings', 'mustuse-manager') . '">' . esc_html__('Settings', 'mustuse-manager') . '</a>',
		];

		// Merge the new links with existing actions and return
		return array_merge($actions, $mylinks);
	}

	/**
	 * Adds a "Donate" link to the plugin's row meta data and removes the Plugin URI.
	 * This function removes the Plugin URI link from the plugin's meta information
	 *
	 * @param array $plugin_meta The existing array of plugin meta links.
	 * @param string $file The plugin file.
	 * @return array The modified array of plugin links.
	 */
	public function plugin_row_meta($plugin_meta, $file)
	{
		// Check if the current file is the plugin's file
		if (plugin_basename(__FILE__) === $file) {
			// Remove Plugin URI from the meta links
			foreach ($plugin_meta as $key => $meta_link) {
				if (strpos($meta_link, esc_url(get_plugin_data(__FILE__)['PluginURI'])) !== false) {
					unset($plugin_meta[$key]); // Remove the Plugin URI link
					break;
				}
			}

			// Add a "Donate" link to the plugin's meta data
			$plugin_meta[] = '<a href="' . esc_url('https://paypal.me/SalvatoreN') . '" target="_blank" rel="noopener noreferrer">' . esc_html__('Donate', 'mustuse-manager') . ' 🎁</a>';
		}

		// Return the updated array of links
		return array_values($plugin_meta);
	}

	/**
	 * Customizes the footer text when on the plugin's admin page.
	 * This function modifies the admin footer text on the MustUse Manager page.
	 *
	 * @param string $footer_text The default footer text.
	 * @return string The modified footer text.
	 */
	public function custom_footer_text($footer_text)
	{
		// Check if the current screen is related to MustUse Manager
		$screen = get_current_screen(); // Get the current admin screen object
		if ($screen && strpos($screen->id, 'mustuse-manager') !== false) {
			// Customize the footer text with the plugin name, version, and author
			$plugin_name = esc_html($this->plugin_data['Name']);
			$plugin_version = esc_html($this->plugin_data['Version']);
			$author = $this->plugin_data['Author'];
			return "<strong><em>$plugin_name v$plugin_version | by $author</em></strong>";
		}

		// Return the default footer text if not on the plugin's admin page
		return $footer_text;
	}
	
	/**
	 * Adds an admin notice or notices.
	 * This private function adds admin notices to be displayed.
	 *
	 * @param string|array $messages The message(s) to be displayed.
	 * @param string $type The type of notice (info, success, warning, error).
	 */
	private function add_admin_notice($messages, $type = 'info')
	{
		// Retrieve current notices from the transient, or set an empty array if none exist
		$current_notices = get_transient('mustuse_manager_notice') ?: [];

		// If the message is not an array, convert it to an array
		if (!is_array($messages)) {
			$messages = [$messages];
		}

		// Add each message to the current notices array with the specified type
		foreach ($messages as $message) {
			$current_notices[] = ['message' => $message, 'type' => $type];
		}

		// Store the updated notices array in the transient
		set_transient('mustuse_manager_notice', $current_notices);
	}

	/**
	 * Displays all admin notices.
	 * This function outputs all stored admin notices.
	 */
	private function display_admin_notices()
	{
		// Retrieve stored notices from the transient
		$notices = get_transient('mustuse_manager_notice');

		// If notices exist, loop through them and display each one
		if ($notices) {
			foreach ($notices as $notice) {
				// Display each notice with its corresponding type and message
				echo '<div class="notice notice-' . esc_attr($notice['type']) . ' is-dismissible">';
				// Escaping applied again for additional security
				echo '<p>' . esc_html($notice['message']) . '</p>';
				echo '</div>';
			}

			// Clear the transient after displaying the notices
			delete_transient('mustuse_manager_notice');
		}
	}
}

/**
 * Initialize MustUSE_Manager class
 */ 
new MustUSE_Manager();
